"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useQueryStore = exports.QueryStoreProvider = exports.fakeRouter = exports.QueryStoreContext = void 0;
const router_1 = require("next/router");
const react_1 = __importStar(require("react"));
const debug_1 = __importDefault(require("debug"));
const lodash_1 = __importDefault(require("lodash"));
const events_1 = require("events");
const store_1 = require("./store");
const debug = debug_1.default('deepcase:store:use-store-query');
const capacitorStorageEvent = new events_1.EventEmitter();
capacitorStorageEvent.setMaxListeners(0);
exports.QueryStoreContext = react_1.createContext(store_1.defaultContext);
exports.fakeRouter = {};
exports.QueryStoreProvider = ({ context = exports.QueryStoreContext, children, }) => {
    const router = router_1.useRouter();
    const { query, pathname, push } = router || exports.fakeRouter;
    const _routerRef = react_1.useRef({});
    _routerRef.current = { query, pathname };
    const _renderingRef = react_1.useRef({});
    const _timeoutRef = react_1.useRef();
    const _cacheRef = react_1.useRef();
    const _getDefaultValue = react_1.useRef();
    _getDefaultValue.current = (key, defaultValue) => {
        if (query && query.hasOwnProperty(key)) {
            try {
                return JSON.parse(query[key]);
            }
            catch (error) {
                return query[key];
            }
        }
        return defaultValue;
    };
    react_1.useEffect(() => {
        _renderingRef.current = {};
        lodash_1.default.each(router === null || router === void 0 ? void 0 : router.query, (value, key) => {
            var _a;
            if (!lodash_1.default.isEqual(value, (_a = _cacheRef === null || _cacheRef === void 0 ? void 0 : _cacheRef.current) === null || _a === void 0 ? void 0 : _a[key])) {
                if (value) {
                    try {
                        capacitorStorageEvent.emit(key, JSON.parse(value));
                    }
                    catch (error) {
                        capacitorStorageEvent.emit(key, value);
                    }
                }
                else {
                    capacitorStorageEvent.emit(key, undefined);
                }
            }
        });
        _cacheRef.current = router === null || router === void 0 ? void 0 : router.query;
    }, [router === null || router === void 0 ? void 0 : router.query]);
    const [useStoreValue] = react_1.useState(() => {
        function useStore(key, defaultValue) {
            const [state, setState] = react_1.useState(_getDefaultValue.current(key, defaultValue));
            const memoDefaultValue = react_1.useMemo(() => defaultValue, []);
            react_1.useEffect(() => {
                const fn = (value) => {
                    setState(value);
                };
                capacitorStorageEvent.on(key, fn);
                return () => {
                    capacitorStorageEvent.off(key, fn);
                };
            }, []);
            const setValue = (value) => {
                try {
                    _renderingRef.current[key] = JSON.stringify(value);
                    clearTimeout(_timeoutRef.current);
                    _timeoutRef.current = setTimeout(() => {
                        var _a, _b;
                        push({
                            pathname: (_a = _routerRef.current) === null || _a === void 0 ? void 0 : _a.pathname,
                            query: Object.assign(Object.assign({}, (_b = _routerRef.current) === null || _b === void 0 ? void 0 : _b.query), _renderingRef.current),
                        });
                    }, 0);
                }
                catch (error) {
                    debug('setStore:error', { error, key, defaultValue: memoDefaultValue, value });
                }
            };
            const [unsetValue] = react_1.useState(() => () => {
                try {
                    if (query[key])
                        delete query[key];
                    clearTimeout(_timeoutRef.current);
                    _timeoutRef.current = setTimeout(() => {
                        var _a, _b;
                        const _q = Object.assign(Object.assign({}, (_a = _routerRef.current) === null || _a === void 0 ? void 0 : _a.query), _renderingRef.current);
                        delete _q[key];
                        push({
                            pathname: (_b = _routerRef.current) === null || _b === void 0 ? void 0 : _b.pathname,
                            query: _q,
                        });
                    }, 0);
                    _renderingRef.current[key] = undefined;
                }
                catch (error) {
                    debug('unsetStore:error', { error, key });
                }
            });
            return [state, setValue, unsetValue];
        }
        ;
        return { useStore };
    });
    return react_1.default.createElement(context.Provider, { value: useStoreValue }, children);
};
function useQueryStore(key, defaultValue, context = exports.QueryStoreContext) {
    return store_1.useStore(key, defaultValue, context);
}
exports.useQueryStore = useQueryStore;
//# sourceMappingURL=query.js.map